@@ -3,35 +3,21 @@
 
 /*------objective function---------------------------------------*/
 
-double evaluate(long *l_nfeval, SEXP par, SEXP fcall, SEXP env)
-{
-   SEXP sexp_fvec, fn;
-   double f_result;
-
-   PROTECT(fn = lang3(fcall, par, R_DotsSymbol));
-      (*l_nfeval)++;  /* increment function evaluation count */
-
-   PROTECT(sexp_fvec = eval(fn, env));
-   f_result = NUMERIC_POINTER(sexp_fvec)[0];
-
-   if(ISNAN(f_result))
-     error("NaN value of objective function! \nPerhaps adjust the bounds.");
-
-   UNPROTECT(2);
-   return(f_result);
-}
-
 SEXP popEvaluate(long *l_nfeval, SEXP parMat, SEXP fcall, SEXP env, int incrementEval)
 {
    SEXP sexp_fvec, fn;
    double *d_result;
    int P = 0;
+   int nr;
 
+   if (isNull(fcall))
+    return parMat;
    PROTECT(fn = lang3(fcall, parMat, R_DotsSymbol)); P++;
-      (*l_nfeval)++;  /* increment function evaluation count */
-
    PROTECT(sexp_fvec = eval(fn, env)); P++;
-   int nr = nrows(sexp_fvec);
+   nr = nrows(sexp_fvec);
+   if(incrementEval)
+     (*l_nfeval) += nr;  
+   
    if(nr != nrows(parMat))
      error("objective function result has different length than parameter matrix");
    switch(TYPEOF(sexp_fvec)) {
